% program frac_mod
% Usage: frac_mod

% Author: D. Eaton September 2, 2014
%
% Copyright 2018, David W. Eaton, University of Calgary, all rights reserved
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER "AS IS" AND ANY EXPRESS OR 
% IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
% MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
% SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
% INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
% TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
% BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
% STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
% OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

% This program accompanies the textbook "Passive Seismic Monitoring of
% Induced Seismicity: Fundamental Principles and Application to Energy
% Technologies", Cambridge University Press
%
% Please refer to the online manual for detailed usage instructions

close all;

% Input dialog for medium parameters. See Shapiro et al. PAGEOPH 2003

prompt = {'Porosity in %','Permeability in microDarcies','K_d in GPa','K_g in GPa','K_f in GPa','Viscosity in Pa-s','Dry frame modulus in GPa'};
name='Poroelastic parameters';
numlines=1;
defaultanswer={'10','50','49','75','2.2','0.00019','22.5'};
options.Resize='on';
options.WindowStyle='normal';
options.Interpreter='tex';
answer=inputdlg(prompt,name,numlines,defaultanswer,options);

phi = str2double(answer(1))/100;
K = str2double(answer(2))*9.869233e-13/1e6;
Kd = str2double(answer(3))*1e9;
Kg = str2double(answer(4))*1e9;
Kf = str2double(answer(5))*1e9;
nu = str2double(answer(6));
mud = str2double(answer(7))*1e9;

% Calculate poroelastic parameter, N

Pd = Kd + 4*mud/3;
alpha = 1-Kd/Kg;
M = 1/((phi/Kf)+(alpha-phi)/Kg);
H = Pd + alpha^2*M;
N = M*Pd/H;

% Input basic fracturing parameters, for illustration only

prompt = {'Frac max height in m','Frac half length in m','Injection rate m^3/min','Injection net pressure in MPa','Injection duration in minutes','Computation duration in minutes'};
name='Injection parameters';
numlines=1;
defaultanswer={'100','200','10','5','60','90'};
options.Resize='on';
options.WindowStyle='normal';
options.Interpreter='tex';
answer=inputdlg(prompt,name,numlines,defaultanswer,options);

Hmax = str2double(answer(1));
L = str2double(answer(2))*2;
R = str2double(answer(3))/60;
Pinject = str2double(answer(4))*1e6;
tinject = str2double(answer(5))*60;
tcalc = str2double(answer(6))*60;

% Total injected volume
V = R*tinject;

% Effective width of fracture
w = V/(L*Hmax);

% time to achieve full frac height
t1 = Hmax^2*w/R;

if t1 > tinject;
   error('Injection time too short to achieve full height growth, please adjust parameters');
end

prompt = {'Vertical stress in MPa','Shmax in MPa','Shmin in MPa','x-component of n','y-component of n','z-component of n','Coefficient of friction'};
name='CFF parameters';
numlines=1;
defaultanswer={'50','60','40','1','1','0','0.6'};
options.Resize='on';
options.WindowStyle='normal';
options.Interpreter='tex';
answer=inputdlg(prompt,name,numlines,defaultanswer,options);

Sv = str2double(answer(1))*1e6;
Shmax = str2double(answer(2))*1e6;
Shmin = str2double(answer(3))*1e6;
n1 = str2double(answer(4));
n2 = str2double(answer(5));
n3 = str2double(answer(6));
frcoef = str2double(answer(7));

sbackground = zeros(3,3);
sbackground(1,1) = Shmin;
sbackground(2,2) = Shmax;
sbackground(3,3) = Sv;
n = [n1,n2,n3];
n = n/norm(n);

p = zeros(500,2000);

Dbar = N*K/nu

D1 = p + Dbar;
D1 = D1(2:499,2:1999);

D2 = D1;

dx = 1.0;
dt = 5.0;

Nsave = 100;

Nmax = tcalc/dt
Ninject = tinject/dt

disp('Computing pore pressure diffusion');

for i = 1:Nmax;
   
    t = (i-1)*dt;
    vt = R*t;
    
   if i <= Ninject; 
      if t < t1;
          h = sqrt(vt/w);
      else
          h = Hmax;
      end
      i1 = 250 - round(h*0.5/dx);
      i2 = i1 + round(h/dx);
      p(i1:i2,1000) = Pinject;
   end
   
   pn = 0*p;

   dp2dx = (p(2:499,3:2000) - 2*p(2:499,2:1999) + p(2:499,1:1998))/dx^2;
   
   dp2dy = (p(3:500,2:1999) - 2*p(2:499,2:1999) + p(1:498,2:1999))/dx^2;
   
   
   dpdt = D1.*dp2dx + D2.*dp2dy;
   
   pn(2:499,2:1999) = p(2:499,2:1999) + dpdt*dt;
       
   if mod((i-1),Nsave) == 0;
       i
       %filename = strcat('p_',num2str(i));
       %save(filename,'pn');
       close all;
       imagesc(pn);
       colorbar vert;
       title(num2str(t/60));
       pause(1);
   end
   
   p = pn;

end

disp('Computing internal deformation');

zinject = 2500;

m = [L, Hmax, zinject+Hmax/2, 90, 0, 0, 0, 0, 0, w];
            
m = m';

k = 1;
for i = 1:200;
    for j = 1:50;
        x(1,k) = (i-1)*10 - 1000;
        x(2,k) = 0;
        x(3,k) = -1*(zinject - 250 + (j-1)*10);
        k = k + 1;
    end
end

mu = Kd;
Poisson = 0.25;

[U D S flag] = disloc3dpm(m,x,mu,Poisson);

pn = pn(1:10:500,1:10:2000);

disp('Computing CFF');

for i = 1:200;
    for j = 1:50;
        
        % build stress tensor
        
        s(1,1) = S(1,(i-1)*50+j);
        s(1,2) = S(2,(i-1)*50+j);
        s(2,1) = s(1,2);
        s(1,3) = S(3,(i-1)*50+j);
        s(3,1) = s(1,3);
        s(2,2) = S(4,(i-1)*50+j);
        s(2,3) = S(5,(i-1)*50+j);
        s(3,2) = s(2,3);
        s(3,3) = S(6,(i-1)*50+j);
        
        s = s + sbackground;
        
        % compute CFF

        T = n*sbackground;
        sigma_n = T*n';
        Tnorm = norm(T);
        tau = sqrt(Tnorm^2-sigma_n^2);
        CFF0 = abs(tau) - frcoef*(sigma_n);
        
        CFF1 = abs(tau) - frcoef*(sigma_n-pn(j,i));  % Pore pressure alone
                
        T = n*s;
        sigma_n = T*n';
        Tnorm = norm(T);
        tau = sqrt(Tnorm^2-sigma_n^2);

        CFF2 = abs(tau) - frcoef*(sigma_n);  % Stress change alone
        
        CFF3 = abs(tau) - frcoef*(sigma_n-pn(j,i));  % Both pore pressure and stress change
        
        DCFF1(j,i) = CFF1 - CFF0;
        DCFF2(j,i) = CFF2 - CFF0;
        DCFF3(j,i) = CFF3 - CFF0;
        
        
        
    end
end

close all;

pcolor([10:10:2000],[10:10:500]-250,DCFF1);
shading flat;
xlabel('Distance in m','FontSize',14);
ylabel('Heigth above injection level in m','FontSize',14);
title('\Delta CFF in Pa due to pore pressure alone','FontSize',14);
colorbar vert;

figure;
pcolor([10:10:2000],[10:10:500]-250,DCFF2);
shading flat;
xlabel('Distance in m','FontSize',14);
ylabel('Heigth above injection level in m','FontSize',14);
title('Delta CFF in Pa due to stress change alone','FontSize',14);
colorbar vert;


figure;
pcolor([10:10:2000],[10:10:500]-250,DCFF3);
shading flat;
xlabel('Distance in m','FontSize',14);
ylabel('Heigth above injection level in m','FontSize',14);
title('\Delta CFF in Pa due to both pore pressure and stress change','FontSize',14);
colorbar vert;


